﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("Packing Sequence Encoding", "Represents an encoding for a bin packing problem using permutation of item-indexes to define the packing sequence.")]
  [StorableClass]
  public class PackingSequenceEncoding : Item, IPackingSolutionEncoding {

    [Storable]
    public Permutation PackingSequence { get; set; }

    [StorableConstructor]
    protected PackingSequenceEncoding(bool deserializing) : base(deserializing) { }
    protected PackingSequenceEncoding(PackingSequenceEncoding original, Cloner cloner)
      : base(original, cloner) {
      this.PackingSequence = cloner.Clone(original.PackingSequence);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PackingSequenceEncoding(this, cloner);
    }

    public PackingSequenceEncoding()
      : base() {
      PackingSequence = new Permutation();
    }

    public override string ToString() {
      StringBuilder sb = new StringBuilder();
      sb.Append("[ ");
      sb.Append(PackingSequence.ToString() + " \n");
      sb.Append("]");
      return sb.ToString();
    }


    public override bool Equals(object obj) {
      if (obj.GetType() == typeof(PackingSequenceEncoding))
        return AreEqual(this, obj as PackingSequenceEncoding);

      return false;
    }
    public override int GetHashCode() {
      return PackingSequence.GetHashCode();
    }
    private static bool AreEqual(PackingSequenceEncoding packingSequenceEncoding1, PackingSequenceEncoding packingSequenceEncoding2) {
      return AreEqual(packingSequenceEncoding1.PackingSequence, packingSequenceEncoding2.PackingSequence);
    }

    private static bool AreEqual(Permutation p1, Permutation p2) {
      if (p1.Length != p2.Length)
        return false;
      for (int i = 0; i < p1.Length; i++) {
        if (p1[i] != p2[i])
          return false;
      }
      return true;
    }
  }
}